@@ -20,15 +20,7 @@ import (
 	"tailscale.com/types/logger"
 )
 
-// tunMTU is the MTU we set on tailscale's TUN interface. wireguard-go
-// defaults to 1420 bytes, which only works if the "outer" MTU is 1500
-// bytes. This breaks on DSL connections (typically 1492 MTU) and on
-// GCE (1460 MTU?!).
-//
-// 1280 is the smallest MTU allowed for IPv6, which is a sensible
-// "probably works everywhere" setting until we develop proper PMTU
-// discovery.
-var tunMTU = 1280
+var tunMTU = DefaultMTU
 
 func init() {
 	if mtu, ok := envknob.LookupInt("TS_DEBUG_MTU"); ok {
