@@ -41,11 +41,16 @@ main() {
 		#  - ID: the short name of the OS (e.g. "debian", "freebsd")
 		#  - VERSION_ID: the numeric release version for the OS, if any (e.g. "18.04")
 		#  - VERSION_CODENAME: the codename of the OS release, if any (e.g. "buster")
+		#  - UBUNTU_CODENAME: if it exists, use instead of VERSION_CODENAME
 		. /etc/os-release
 		case "$ID" in
-			ubuntu|pop|neon|zorin|elementary)
+			ubuntu|pop|neon|zorin)
 				OS="ubuntu"
-				VERSION="$VERSION_CODENAME"
+				if [ "${UBUNTU_CODENAME:-}" != "" ]; then
+				    VERSION="$UBUNTU_CODENAME"
+				else
+				    VERSION="$VERSION_CODENAME"
+				fi
 				PACKAGETYPE="apt"
 				# Third-party keyrings became the preferred method of
 				# installation in Ubuntu 20.04.
@@ -85,6 +90,16 @@ main() {
 					APT_KEY_TYPE="keyring"
 				fi
 				;;
+			elementary)
+				OS="ubuntu"
+				VERSION="$UBUNTU_CODENAME"
+				PACKAGETYPE="apt"
+				if [ "$VERSION_ID" -lt 6 ]; then
+					APT_KEY_TYPE="legacy"
+				else
+					APT_KEY_TYPE="keyring"
+				fi
+				;;
 			parrot)
 				OS="debian"
 				PACKAGETYPE="apt"
@@ -161,6 +176,11 @@ main() {
 				VERSION="$VERSION_ID"
 				PACKAGETYPE="yum"
 				;;
+			xenenterprise)
+				OS="centos"
+				VERSION="$(echo "$VERSION_ID" | cut -f1 -d.)"
+				PACKAGETYPE="yum"
+				;;
 			opensuse-leap)
 				OS="opensuse"
 				VERSION="leap/$VERSION_ID"
@@ -294,7 +314,8 @@ main() {
 			fi
 		;;
 		rhel)
-			if [ "$VERSION" != "8" ]
+			if [ "$VERSION" != "8" ] && \
+                           [ "$VERSION" != "9" ]
 			then
 				OS_UNSUPPORTED=1
 			fi
