@@ -21,9 +21,12 @@ import (
 	"path/filepath"
 	"reflect"
 	"runtime"
+	"strconv"
 	"strings"
+	"sync"
 	"time"
 
+	"go4.org/mem"
 	"inet.af/netaddr"
 	"tailscale.com/envknob"
 	"tailscale.com/metrics"
@@ -87,6 +90,29 @@ func AllowDebugAccess(r *http.Request) bool {
 	return false
 }
 
+// AcceptsEncoding reports whether r accepts the named encoding
+// ("gzip", "br", etc).
+func AcceptsEncoding(r *http.Request, enc string) bool {
+	h := r.Header.Get("Accept-Encoding")
+	if h == "" {
+		return false
+	}
+	if !strings.Contains(h, enc) && !mem.ContainsFold(mem.S(h), mem.S(enc)) {
+		return false
+	}
+	remain := h
+	for len(remain) > 0 {
+		var part string
+		part, remain, _ = strings.Cut(remain, ",")
+		part = strings.TrimSpace(part)
+		part, _, _ = strings.Cut(part, ";")
+		if part == enc {
+			return true
+		}
+	}
+	return false
+}
+
 // Protected wraps a provided debug handler, h, returning a Handler
 // that enforces AllowDebugAccess and returns forbidden replies for
 // unauthorized requests.
@@ -136,7 +162,7 @@ func (h Port80Handler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	}
 	host := h.FQDN
 	if host == "" {
-		host = r.URL.Hostname()
+		host = r.Host
 	}
 	target := "https://" + host + path
 	http.Redirect(w, r, target, http.StatusFound)
@@ -167,6 +193,10 @@ type HandlerOptions struct {
 	// of status codes for handled responses.
 	// The keys are "1xx", "2xx", "3xx", "4xx", and "5xx".
 	StatusCodeCounters *expvar.Map
+	// If non-nil, StatusCodeCountersFull maintains counters of status
+	// codes for handled responses.
+	// The keys are HTTP numeric response codes e.g. 200, 404, ...
+	StatusCodeCountersFull *expvar.Map
 }
 
 // ReturnHandlerFunc is an adapter to allow the use of ordinary
@@ -273,11 +303,38 @@ func (h retHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	}
 
 	if h.opts.StatusCodeCounters != nil {
-		key := fmt.Sprintf("%dxx", msg.Code/100)
-		h.opts.StatusCodeCounters.Add(key, 1)
+		h.opts.StatusCodeCounters.Add(responseCodeString(msg.Code/100), 1)
+	}
+
+	if h.opts.StatusCodeCountersFull != nil {
+		h.opts.StatusCodeCountersFull.Add(responseCodeString(msg.Code), 1)
 	}
 }
 
+func responseCodeString(code int) string {
+	if v, ok := responseCodeCache.Load(code); ok {
+		return v.(string)
+	}
+
+	var ret string
+	if code < 10 {
+		ret = fmt.Sprintf("%dxx", code)
+	} else {
+		ret = strconv.Itoa(code)
+	}
+	responseCodeCache.Store(code, ret)
+	return ret
+}
+
+// responseCodeCache memoizes the string form of HTTP response codes,
+// so that the hot request-handling codepath doesn't have to allocate
+// in strconv/fmt for every request.
+//
+// Keys are either full HTTP response code ints (200, 404) or "family"
+// ints representing entire families (e.g. 2 for 2xx codes). Values
+// are the string form of that code/family.
+var responseCodeCache sync.Map
+
 // loggingResponseWriter wraps a ResponseWriter and record the HTTP
 // response code that gets sent, if any.
 type loggingResponseWriter struct {
