@@ -552,3 +552,73 @@ func (promWriter) WritePrometheus(w io.Writer, prefix string) {
 func (promWriter) String() string {
 	return ""
 }
+
+func TestAcceptsEncoding(t *testing.T) {
+	tests := []struct {
+		in, enc string
+		want    bool
+	}{
+		{"", "gzip", false},
+		{"gzip", "gzip", true},
+		{"foo,gzip", "gzip", true},
+		{"foo, gzip", "gzip", true},
+		{"foo, gzip ", "gzip", true},
+		{"gzip, foo ", "gzip", true},
+		{"gzip, foo ", "br", false},
+		{"gzip, foo ", "fo", false},
+		{"gzip;q=1.2, foo ", "gzip", true},
+		{" gzip;q=1.2, foo ", "gzip", true},
+	}
+	for i, tt := range tests {
+		h := make(http.Header)
+		if tt.in != "" {
+			h.Set("Accept-Encoding", tt.in)
+		}
+		got := AcceptsEncoding(&http.Request{Header: h}, tt.enc)
+		if got != tt.want {
+			t.Errorf("%d. got %v; want %v", i, got, tt.want)
+		}
+	}
+}
+
+func TestPort80Handler(t *testing.T) {
+	tests := []struct {
+		name    string
+		h       *Port80Handler
+		req     string
+		wantLoc string
+	}{
+		{
+			name:    "no_fqdn",
+			h:       &Port80Handler{},
+			req:     "GET / HTTP/1.1\r\nHost: foo.com\r\n\r\n",
+			wantLoc: "https://foo.com/",
+		},
+		{
+			name:    "fqdn_and_path",
+			h:       &Port80Handler{FQDN: "bar.com"},
+			req:     "GET /path HTTP/1.1\r\nHost: foo.com\r\n\r\n",
+			wantLoc: "https://bar.com/path",
+		},
+		{
+			name:    "path_and_query_string",
+			h:       &Port80Handler{FQDN: "baz.com"},
+			req:     "GET /path?a=b HTTP/1.1\r\nHost: foo.com\r\n\r\n",
+			wantLoc: "https://baz.com/path?a=b",
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			r, _ := http.ReadRequest(bufio.NewReader(strings.NewReader(tt.req)))
+			rec := httptest.NewRecorder()
+			tt.h.ServeHTTP(rec, r)
+			got := rec.Result()
+			if got, want := got.StatusCode, 302; got != want {
+				t.Errorf("got status code %v; want %v", got, want)
+			}
+			if got, want := got.Header.Get("Location"), "https://foo.com/"; got != tt.wantLoc {
+				t.Errorf("Location = %q; want %q", got, want)
+			}
+		})
+	}
+}
